@@ -19,19 +19,6 @@
  */
 void do_cmd_inven(void)
 {
-	char out_val[160];
-
-
-	/* Note that we are in "inventory" mode */
-	command_wrk = FALSE;
-
-#ifdef ALLOW_EASY_FLOOR
-
-	/* Note that we are in "inventory" mode */
-	if (easy_floor) command_wrk = (USE_INVEN);
-
-#endif /* ALLOW_EASY_FLOOR */
-
 	/* Save screen */
 	screen_save();
 
@@ -39,38 +26,28 @@ void do_cmd_inven(void)
 	item_tester_full = TRUE;
 
 	/* Display the inventory */
-	show_inven();
+	show_list(p_ptr->inventory);
 
 	/* Hack -- hide empty slots */
 	item_tester_full = FALSE;
 
-	sprintf(out_val, "Inventory: carrying %d.%d pounds (%d%% of capacity). Command: ",
-	    p_ptr->total_weight / 10, p_ptr->total_weight % 10,
-	    (p_ptr->total_weight * 100) / ((adj_str_wgt[p_ptr->stat_ind[A_STR]] * 100) / 2));
-
 	/* Get a command */
-	prt(out_val, 0, 0);
+	prtf(0, 0, "Inventory: carrying %d.%d pounds (%d%% of capacity). Command: ",
+			p_ptr->total_weight / 10, p_ptr->total_weight % 10,
+			(p_ptr->total_weight * 100) /
+			((adj_str_wgt[p_ptr->stat[A_STR].ind] * 100) / 2));
 
 	/* Get a new command */
-	command_new = inkey();
+	p_ptr->cmd.new = inkey();
 
 	/* Load screen */
 	screen_load();
 
-
 	/* Process "Escape" */
-	if (command_new == ESCAPE)
+	if (p_ptr->cmd.new == ESCAPE)
 	{
 		/* Reset stuff */
-		command_new = 0;
-		command_gap = 50;
-	}
-
-	/* Process normal keys */
-	else
-	{
-		/* Hack -- Use "display" mode */
-		command_see = TRUE;
+		p_ptr->cmd.new = 0;
 	}
 }
 
@@ -80,19 +57,6 @@ void do_cmd_inven(void)
  */
 void do_cmd_equip(void)
 {
-	char out_val[160];
-
-
-	/* Note that we are in "equipment" mode */
-	command_wrk = TRUE;
-
-#ifdef ALLOW_EASY_FLOOR
-
-	/* Note that we are in "equipment" mode */
-	if (easy_floor) command_wrk = (USE_EQUIP);
-
-#endif /* ALLOW_EASY_FLOOR  */
-
 	/* Save the screen */
 	screen_save();
 
@@ -105,48 +69,24 @@ void do_cmd_equip(void)
 	/* Hack -- undo the hack above */
 	item_tester_full = FALSE;
 
-	/* Build a prompt */
-	sprintf(out_val, "Equipment: carrying %d.%d pounds (%d%% of capacity). Command: ",
-	    p_ptr->total_weight / 10, p_ptr->total_weight % 10,
-	    (p_ptr->total_weight * 100) / ((adj_str_wgt[p_ptr->stat_ind[A_STR]] * 100) / 2));
-
 	/* Get a command */
-	prt(out_val, 0, 0);
+	prtf(0, 0, "Equipment: carrying %d.%d pounds (%d%% of capacity). Command: ",
+			p_ptr->total_weight / 10, p_ptr->total_weight % 10,
+			(p_ptr->total_weight * 100) /
+			((adj_str_wgt[p_ptr->stat[A_STR].ind] * 100) / 2));
 
 	/* Get a new command */
-	command_new = inkey();
+	p_ptr->cmd.new = inkey();
 
 	/* Restore the screen */
 	screen_load();
 
-
 	/* Process "Escape" */
-	if (command_new == ESCAPE)
+	if (p_ptr->cmd.new == ESCAPE)
 	{
 		/* Reset stuff */
-		command_new = 0;
-		command_gap = 50;
+		p_ptr->cmd.new = 0;
 	}
-
-	/* Process normal keys */
-	else
-	{
-		/* Enter "display" mode */
-		command_see = TRUE;
-	}
-}
-
-
-/*
- * The "wearable" tester
- */
-static bool item_tester_hook_wear(object_type *o_ptr)
-{
-	/* Check for a usable slot */
-	if (wield_slot(o_ptr) >= INVEN_WIELD) return (TRUE);
-
-	/* Assume not wearable */
-	return (FALSE);
 }
 
 
@@ -155,17 +95,14 @@ static bool item_tester_hook_wear(object_type *o_ptr)
  */
 void do_cmd_wield(void)
 {
-	int i, item, slot;
+	int slot;
 
-	object_type forge;
 	object_type *q_ptr;
 
 	object_type *o_ptr;
 
 	cptr act;
 
-	char o_name[80];
-
 	cptr q, s;
 
 	/* Restrict the choices */
@@ -174,120 +111,86 @@ void do_cmd_wield(void)
 	/* Get an item */
 	q = "Wear/Wield which item? ";
 	s = "You have nothing you can wear or wield.";
-	if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
 
-	/* Get the item (in the pack) */
-	if (item >= 0)
-	{
-		o_ptr = &inventory[item];
-	}
-
-	/* Get the item (on the floor) */
-	else
-	{
-		o_ptr = &o_list[0 - item];
-	}
+	q_ptr = get_item(q, s, (USE_INVEN | USE_FLOOR));
 
+	/* Not a valid item */
+	if (!q_ptr) return;
 
 	/* Check the slot */
-	slot = wield_slot(o_ptr);
+	slot = wield_slot(q_ptr);
+
+	/* Hack - rings are special */
+	if (slot == EQUIP_LEFT)
+	{
+		/* Check to see if have two rings */
+		if ((p_ptr->equipment[EQUIP_LEFT].k_idx) &&
+			(p_ptr->equipment[EQUIP_RIGHT].k_idx))
+		{
+			if (get_check("Use right hand? ")) slot = EQUIP_RIGHT;
+		}
+	}
+
+	/* Access the wield slot */
+	o_ptr = &p_ptr->equipment[slot];
 
 	/* Prevent wielding into a cursed slot */
-	if (cursed_p(&inventory[slot]))
+	if (cursed_p(o_ptr))
 	{
-		/* Describe it */
-		object_desc(o_name, &inventory[slot], FALSE, 0);
-
 		/* Message */
-		msg_format("The %s you are %s appears to be cursed.",
-		           o_name, describe_use(slot));
+		msgf("The %v you are %s appears to be cursed.",
+			 OBJECT_FMT(o_ptr, FALSE, 0), describe_use(slot));
 
 		/* Cancel the command */
 		return;
 	}
 
-	if (cursed_p(o_ptr) && wear_confirm &&
-	    (object_known_p(o_ptr) || (o_ptr->ident & IDENT_SENSE)))
+	if (cursed_p(q_ptr) && confirm_wear &&
+		(object_known_p(q_ptr) || (q_ptr->info & OB_SENSE)))
 	{
-		char dummy[512];
-
-		/* Describe it */
-		object_desc(o_name, o_ptr, FALSE, 0);
-
-		sprintf(dummy, "Really use the %s {cursed}? ", o_name);
-
-		if (!get_check(dummy))
+		if (!get_check("Really use the %v {cursed}? ",
+			 OBJECT_FMT(q_ptr, FALSE, 0)))
 			return;
 	}
 
-	/* Check if completed a quest */
-	for (i = 0; i < max_quests; i++)
-	{
-		if ((quest[i].type == QUEST_TYPE_FIND_ARTIFACT) &&
-		    (quest[i].status == QUEST_STATUS_TAKEN) &&
-		    (quest[i].k_idx == o_ptr->name1))
-		{
-			quest[i].status = QUEST_STATUS_COMPLETED;
-			msg_print("You completed your quest!");
-			msg_print(NULL);
-		}
-	}
-
 	/* Take a turn */
-	energy_use = 100;
+	p_ptr->energy_use = 100;
 
-	/* Get local object */
-	q_ptr = &forge;
-
-	/* Obtain local object */
-	object_copy(q_ptr, o_ptr);
-
-	/* Modify quantity */
-	q_ptr->number = 1;
-
-	/* Decrease the item (from the pack) */
-	if (item >= 0)
-	{
-		inven_item_increase(item, -1);
-		inven_item_optimize(item);
-	}
-
-	/* Decrease the item (from the floor) */
-	else
-	{
-		floor_item_increase(0 - item, -1);
-		floor_item_optimize(0 - item);
-	}
-
-	/* Access the wield slot */
-	o_ptr = &inventory[slot];
+	/* Split object */
+	q_ptr = item_split(q_ptr, 1);
 
 	/* Take off existing item */
 	if (o_ptr->k_idx)
 	{
 		/* Take off existing item */
-		(void)inven_takeoff(slot, 255);
+		(void)inven_takeoff(o_ptr);
 	}
 
 	/* Wear the new stuff */
-	object_copy(o_ptr, q_ptr);
+	swap_objects(o_ptr, q_ptr);
 
-	/* Increase the weight */
-	p_ptr->total_weight += q_ptr->weight;
+	/* Forget stack */
+	o_ptr->next_o_idx = 0;
 
-	/* Increment the equip counter by hand */
-	equip_cnt++;
+	/* Forget location */
+	o_ptr->iy = o_ptr->ix = 0;
+
+	/* Forget Region */
+	o_ptr->region = 0;
+
+	/* Now no longer allocated in o_list[] */
+	o_ptr->allocated = FALSE;
 
 	/* Where is the item now */
-	if (slot == INVEN_WIELD)
+	if (slot == EQUIP_WIELD)
 	{
 		act = "You are wielding";
 	}
-	else if (slot == INVEN_BOW)
+	else if (slot == EQUIP_BOW)
 	{
 		act = "You are shooting with";
 	}
-	else if (slot == INVEN_LITE)
+	else if (slot == EQUIP_LITE)
 	{
 		act = "Your light source is";
 	}
@@ -296,24 +199,26 @@ void do_cmd_wield(void)
 		act = "You are wearing";
 	}
 
-	/* Describe the result */
-	object_desc(o_name, o_ptr, TRUE, 3);
-
 	/* Message */
-	msg_format("%s %s (%c).", act, o_name, index_to_label(slot));
+	msgf("%s %v (%c).", act, OBJECT_FMT(o_ptr, TRUE, 3), I2A(slot));
 
 	/* Cursed! */
 	if (cursed_p(o_ptr))
 	{
 		/* Warn the player */
-		msg_print("Oops! It feels deathly cold!");
+		msgf("Oops! It feels deathly cold!");
+
+		chg_virtue(V_HARMONY, -1);
 
 		/* Note the curse */
-		o_ptr->ident |= (IDENT_SENSE);
+		o_ptr->info |= (OB_SENSE);
 	}
 
-	/* Recalculate bonuses */
-	p_ptr->update |= (PU_BONUS);
+	/* Learn some "obvious" things about the item */
+	o_ptr->kn_flags1 |= (o_ptr->flags1 & TR1_EASY_MASK);
+
+	/* Recalculate bonuses and weight */
+	p_ptr->update |= (PU_BONUS | PU_WEIGHT);
 
 	/* Recalculate torch */
 	p_ptr->update |= (PU_TORCH);
@@ -325,6 +230,8 @@ void do_cmd_wield(void)
 
 	/* Window stuff */
 	p_ptr->window |= (PW_INVEN | PW_EQUIP | PW_PLAYER);
+
+	make_noise(1);
 }
 
 
@@ -334,8 +241,6 @@ void do_cmd_wield(void)
  */
 void do_cmd_takeoff(void)
 {
-	int item;
-
 	object_type *o_ptr;
 
 	cptr q, s;
@@ -343,39 +248,30 @@ void do_cmd_takeoff(void)
 	/* Get an item */
 	q = "Take off which item? ";
 	s = "You are not wearing anything to take off.";
-	if (!get_item(&item, q, s, (USE_EQUIP))) return;
 
-	/* Get the item (in the pack) */
-	if (item >= 0)
-	{
-		o_ptr = &inventory[item];
-	}
+	o_ptr = get_item(q, s, (USE_EQUIP));
 
-	/* Get the item (on the floor) */
-	else
-	{
-		o_ptr = &o_list[0 - item];
-	}
+	/* Not a valid item */
+	if (!o_ptr) return;
 
 
 	/* Item is cursed */
 	if (cursed_p(o_ptr))
 	{
 		/* Oops */
-		msg_print("Hmmm, it seems to be cursed.");
+		msgf("Hmmm, it seems to be cursed.");
 
 		/* Nope */
 		return;
 	}
 
-
 	/* Take a partial turn */
-	energy_use = 50;
+	p_ptr->energy_use = 50;
 
 	/* Take off the item */
-	(void)inven_takeoff(item, 255);
+	(void)inven_takeoff(o_ptr);
 
-	p_ptr->redraw |= (PR_EQUIPPY);
+	make_noise(1);
 }
 
 
@@ -384,7 +280,7 @@ void do_cmd_takeoff(void)
  */
 void do_cmd_drop(void)
 {
-	int item, amt = 1;
+	int amt = 1;
 
 	object_type *o_ptr;
 
@@ -393,26 +289,17 @@ void do_cmd_drop(void)
 	/* Get an item */
 	q = "Drop which item? ";
 	s = "You have nothing to drop.";
-	if (!get_item(&item, q, s, (USE_EQUIP | USE_INVEN))) return;
 
-	/* Get the item (in the pack) */
-	if (item >= 0)
-	{
-		o_ptr = &inventory[item];
-	}
-
-	/* Get the item (on the floor) */
-	else
-	{
-		o_ptr = &o_list[0 - item];
-	}
+	o_ptr = get_item(q, s, (USE_EQUIP | USE_INVEN));
 
+	/* Not a valid item */
+	if (!o_ptr) return;
 
 	/* Hack -- Cannot remove cursed items */
-	if ((item >= INVEN_WIELD) && cursed_p(o_ptr))
+	if ((!o_ptr->allocated) && cursed_p(o_ptr))
 	{
 		/* Oops */
-		msg_print("Hmmm, it seems to be cursed.");
+		msgf("Hmmm, it seems to be cursed.");
 
 		/* Nope */
 		return;
@@ -431,23 +318,24 @@ void do_cmd_drop(void)
 
 
 	/* Take a partial turn */
-	energy_use = 50;
+	p_ptr->energy_use = 50;
 
 	/* Drop (some of) the item */
-	inven_drop(item, amt);
+	inven_drop(o_ptr, amt);
 
 	p_ptr->redraw |= (PR_EQUIPPY);
+
+	make_noise(1);
 }
 
 
-static bool high_level_book(object_type *o_ptr)
+static bool high_level_book(const object_type *o_ptr)
 {
 	if ((o_ptr->tval == TV_LIFE_BOOK) ||
-	    (o_ptr->tval == TV_SORCERY_BOOK) ||
-	    (o_ptr->tval == TV_NATURE_BOOK) ||
-	    (o_ptr->tval == TV_CHAOS_BOOK) ||
-	    (o_ptr->tval == TV_DEATH_BOOK) ||
-	    (o_ptr->tval == TV_TRUMP_BOOK))
+		(o_ptr->tval == TV_SORCERY_BOOK) ||
+		(o_ptr->tval == TV_NATURE_BOOK) ||
+		(o_ptr->tval == TV_CHAOS_BOOK) ||
+		(o_ptr->tval == TV_DEATH_BOOK) || (o_ptr->tval == TV_TRUMP_BOOK))
 	{
 		if (o_ptr->sval > 1)
 			return TRUE;
@@ -459,129 +347,36 @@ static bool high_level_book(object_type *o_ptr)
 }
 
 
-/*
- * Destroy an item
- */
-void do_cmd_destroy(void)
+bool destroy_item_aux(object_type *o_ptr, int amt)
 {
-	int			item, amt = 1;
-	int			old_number;
+	bool gain_expr = FALSE;
 
-	bool		force = FALSE;
-
-	object_type		*o_ptr;
-
-	char		o_name[80];
-
-	char		out_val[160];
-
-	cptr q, s;
-
-	/* Hack -- force destruction */
-	if (command_arg > 0) force = TRUE;
-
-
-	/* Get an item */
-	q = "Destroy which item? ";
-	s = "You have nothing to destroy.";
-	if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
-
-	/* Get the item (in the pack) */
-	if (item >= 0)
+	/* Can the player destroy the object? */
+	if (!can_player_destroy_object(o_ptr))
 	{
-		o_ptr = &inventory[item];
-	}
-
-	/* Get the item (on the floor) */
-	else
-	{
-		o_ptr = &o_list[0 - item];
-	}
-
-
-	/* See how many items */
-	if (o_ptr->number > 1)
-	{
-		/* Get a quantity */
-		amt = get_quantity(NULL, o_ptr->number);
-
-		/* Allow user abort */
-		if (amt <= 0) return;
-	}
-
-
-	/* Describe the object */
-	old_number = o_ptr->number;
-	o_ptr->number = amt;
-	object_desc(o_name, o_ptr, TRUE, 3);
-	o_ptr->number = old_number;
-
-	/* Verify unless quantity given */
-	if (!force)
-	{
-		if (!(auto_destroy && (object_value(o_ptr) < 1)))
-		{
-			/* Make a verification */
-			sprintf(out_val, "Really destroy %s? ", o_name);
-			if (!get_check(out_val)) return;
-		}
-	}
-
-#ifdef USE_SCRIPT
-
-	if (destroy_object_callback(o_ptr, amt)) return;
-
-#endif /* USE_SCRIPT */
-
-	/* Take a turn */
-	energy_use = 100;
-
-	/* Artifacts cannot be destroyed */
-	if (artifact_p(o_ptr) || o_ptr->art_name)
-	{
-		cptr feel = "special";
-
-		energy_use = 0;
-
 		/* Message */
-		msg_format("You cannot destroy %s.", o_name);
-
-		/* Hack -- Handle icky artifacts */
-		if (cursed_p(o_ptr) || broken_p(o_ptr)) feel = "terrible";
-
-		/* Hack -- inscribe the artifact */
-		o_ptr->note = quark_add(feel);
-
-		/* We have "felt" it (again) */
-		o_ptr->ident |= (IDENT_SENSE);
-
-		/* Combine the pack */
-		p_ptr->notice |= (PN_COMBINE);
-
-		p_ptr->redraw |= (PR_EQUIPPY);
-
-		/* Window stuff */
-		p_ptr->window |= (PW_INVEN | PW_EQUIP);
+		msgf("You cannot destroy %v.", OBJECT_FMT(o_ptr, TRUE, 3));
 
 		/* Done */
-		return;
+		return (FALSE);
 	}
 
+	/* Take a turn */
+	p_ptr->energy_use += 100;
+
 	/* Message */
-	msg_format("You destroy %s.", o_name);
+	msgf("You destroy %v.", OBJECT_FMT(o_ptr, TRUE, 3));
 	sound(SOUND_DESTITEM);
 
 	if (high_level_book(o_ptr))
 	{
-		bool gain_expr = FALSE;
-
-		if (p_ptr->pclass == CLASS_WARRIOR)
+		if (p_ptr->rp.pclass == CLASS_WARRIOR)
 		{
 			gain_expr = TRUE;
 		}
-		else if (p_ptr->pclass == CLASS_PALADIN)
+		else if (p_ptr->rp.pclass == CLASS_PALADIN)
 		{
-			if (p_ptr->realm1 == REALM_LIFE)
+			if (p_ptr->spell.r[0].realm == REALM_LIFE)
 			{
 				if (o_ptr->tval != TV_LIFE_BOOK) gain_expr = TRUE;
 			}
@@ -596,39 +391,113 @@ void do_cmd_destroy(void)
 			s32b tester_exp = p_ptr->max_exp / 20;
 			if (tester_exp > 10000) tester_exp = 10000;
 			if (o_ptr->sval < 3) tester_exp /= 4;
-			if (tester_exp<1) tester_exp = 1;
+			if (tester_exp < 1) tester_exp = 1;
 
-			msg_print("You feel more experienced.");
+			msgf("You feel more experienced.");
 			gain_exp(tester_exp * amt);
 		}
+
+		if (high_level_book(o_ptr) && o_ptr->tval == TV_LIFE_BOOK)
+		{
+			chg_virtue(V_UNLIFE, 1);
+			chg_virtue(V_VITALITY, -1);
+		}
+		else if (high_level_book(o_ptr) && o_ptr->tval == TV_DEATH_BOOK)
+		{
+			chg_virtue(V_UNLIFE, -1);
+			chg_virtue(V_VITALITY, 1);
+		}
+
+		if (o_ptr->to_a || o_ptr->to_h || o_ptr->to_d)
+			chg_virtue(V_ENCHANT, -1);
+
+		if (object_value_real(o_ptr) > 30000)
+			chg_virtue(V_SACRIFICE, 2);
+
+		else if (object_value_real(o_ptr) > 10000)
+			chg_virtue(V_SACRIFICE, 1);
 	}
 
-	/*
-	 * Hack -- If rods or wand are destroyed, the total maximum timeout or
-	 * charges of the stack needs to be reduced, unless all the items are
-	 * being destroyed. -LM-
-	 */
-	if (((o_ptr->tval == TV_WAND) || (o_ptr->tval == TV_ROD)) &&
-		(amt < o_ptr->number))
+	if (o_ptr->to_a != 0 || o_ptr->to_d != 0 || o_ptr->to_h != 0)
+		chg_virtue(V_HARMONY, 1);
+
+	make_noise(1);
+
+	/* We destroyed the item(s) */
+	return (TRUE);
+}
+
+
+/*
+ * Destroy an item
+ */
+void do_cmd_destroy(void)
+{
+	int amt = 1;
+	int old_number;
+
+	bool force = FALSE;
+
+	object_type *o_ptr;
+
+	cptr q, s;
+
+	/* Hack -- force destruction */
+	if (p_ptr->cmd.arg > 0) force = TRUE;
+
+
+	/* Get an item */
+	q = "Destroy which item? ";
+	s = "You have nothing to destroy.";
+
+	o_ptr = get_item(q, s, (USE_INVEN | USE_FLOOR));
+
+	/* Not a valid item */
+	if (!o_ptr) return;
+
+	/* See how many items */
+	if (o_ptr->number > 1)
 	{
-		o_ptr->pval -= o_ptr->pval * amt / o_ptr->number;
+		/* Get a quantity */
+		amt = get_quantity(NULL, o_ptr->number);
+
+		/* Allow user abort */
+		if (amt <= 0) return;
 	}
 
-	/* Eliminate the item (from the pack) */
-	if (item >= 0)
-	{
-		inven_item_increase(item, -amt);
-		inven_item_describe(item);
-		inven_item_optimize(item);
-	}
 
-	/* Eliminate the item (from the floor) */
-	else
+	/* Describe the objects to delete */
+	old_number = o_ptr->number;
+	o_ptr->number = amt;
+	
+
+	/* Verify unless quantity given */
+	if (!force)
 	{
-		floor_item_increase(0 - item, -amt);
-		floor_item_describe(0 - item);
-		floor_item_optimize(0 - item);
+		if (!(auto_destroy && (object_value(o_ptr) < 1)))
+		{
+			/* Make a verification */
+			if (!get_check("Really destroy %v? ", OBJECT_FMT(o_ptr, TRUE, 3)))
+			{
+				o_ptr->number = old_number;
+				return;
+			}
+		}
 	}
+	
+	o_ptr->number = old_number;
+
+	/* No energy used yet */
+	p_ptr->energy_use = 0;
+
+	/* Physically try to destroy the item(s) */
+	if (!destroy_item_aux(o_ptr, amt)) return;
+
+	/* Reduce the charges of rods/wands */
+	reduce_charges(o_ptr, amt);
+
+	/* Eliminate the item */
+	item_increase(o_ptr, -amt);
 }
 
 
@@ -637,51 +506,33 @@ void do_cmd_destroy(void)
  */
 void do_cmd_observe(void)
 {
-	int			item;
-
-	object_type		*o_ptr;
-
-	char		o_name[80];
+	object_type *o_ptr;
 
 	cptr q, s;
 
 	/* Get an item */
 	q = "Examine which item? ";
 	s = "You have nothing to examine.";
-	if (!get_item(&item, q, s, (USE_EQUIP | USE_INVEN | USE_FLOOR))) return;
 
-	/* Get the item (in the pack) */
-	if (item >= 0)
-	{
-		o_ptr = &inventory[item];
-	}
+	o_ptr = get_item(q, s, (USE_EQUIP | USE_INVEN | USE_FLOOR));
 
-	/* Get the item (on the floor) */
-	else
-	{
-		o_ptr = &o_list[0 - item];
-	}
-
-
-	/* Require full knowledge */
-	if (!(o_ptr->ident & IDENT_MENTAL))
-	{
-		msg_print("You have no special knowledge about that item.");
-		return;
-	}
-
-
-	/* Description */
-	object_desc(o_name, o_ptr, TRUE, 3);
+	/* Not a valid item */
+	if (!o_ptr) return;
 
 	/* Describe */
-	msg_format("Examining %s...", o_name);
+	msgf("Examining %v...", OBJECT_FMT(o_ptr, TRUE, 3));
 
 	/* Describe it fully */
-	if (!identify_fully_aux(o_ptr)) msg_print("You see nothing special.");
+	if (!identify_fully_aux(o_ptr)) msgf("You see nothing special.");
 }
 
+static bool item_tester_inscribed(const object_type *o_ptr)
+{
+	/* Nothing to remove */
+	if (!o_ptr->inscription) return (FALSE);
 
+	return (TRUE);
+}
 
 /*
  * Remove the inscription from an object
@@ -689,47 +540,35 @@ void do_cmd_observe(void)
  */
 void do_cmd_uninscribe(void)
 {
-	int   item;
-
 	object_type *o_ptr;
 
 	cptr q, s;
+	
+	/* Only inscribed items */
+	item_tester_hook = item_tester_inscribed;
 
 	/* Get an item */
 	q = "Un-inscribe which item? ";
 	s = "You have nothing to un-inscribe.";
-	if (!get_item(&item, q, s, (USE_EQUIP | USE_INVEN | USE_FLOOR))) return;
 
-	/* Get the item (in the pack) */
-	if (item >= 0)
-	{
-		o_ptr = &inventory[item];
-	}
+	o_ptr = get_item(q, s, (USE_EQUIP | USE_INVEN | USE_FLOOR));
 
-	/* Get the item (on the floor) */
-	else
-	{
-		o_ptr = &o_list[0 - item];
-	}
-
-	/* Nothing to remove */
-	if (!o_ptr->note)
-	{
-		msg_print("That item had no inscription to remove.");
-		return;
-	}
+	/* Not a valid item */
+	if (!o_ptr) return;
 
 	/* Message */
-	msg_print("Inscription removed.");
+	msgf("Inscription removed.");
 
-	/* Remove the incription */
-	o_ptr->note = 0;
+    /* Remove the incription */
+    quark_remove(&o_ptr->inscription);
 
 	/* Combine the pack */
 	p_ptr->notice |= (PN_COMBINE);
 
 	/* Window stuff */
 	p_ptr->window |= (PW_INVEN | PW_EQUIP);
+
+	make_noise(2);
 }
 
 
@@ -738,61 +577,49 @@ void do_cmd_uninscribe(void)
  */
 void do_cmd_inscribe(void)
 {
-	int			item;
+	object_type *o_ptr;
 
-	object_type		*o_ptr;
-
-	char		o_name[80];
-
-	char		out_val[80];
+	char out_val[80];
 
 	cptr q, s;
 
 	/* Get an item */
 	q = "Inscribe which item? ";
 	s = "You have nothing to inscribe.";
-	if (!get_item(&item, q, s, (USE_EQUIP | USE_INVEN | USE_FLOOR))) return;
 
-	/* Get the item (in the pack) */
-	if (item >= 0)
-	{
-		o_ptr = &inventory[item];
-	}
+	o_ptr = get_item(q, s, (USE_EQUIP | USE_INVEN | USE_FLOOR));
 
-	/* Get the item (on the floor) */
-	else
-	{
-		o_ptr = &o_list[0 - item];
-	}
-
-	/* Describe the activity */
-	object_desc(o_name, o_ptr, TRUE, 3);
+	/* Not a valid item */
+	if (!o_ptr) return;
 
 	/* Message */
-	msg_format("Inscribing %s.", o_name);
-	msg_print(NULL);
+	msgf("Inscribing %v.", OBJECT_FMT(o_ptr, TRUE, 3));
+	message_flush();
 
 	/* Start with nothing */
-	strcpy(out_val, "");
+	out_val[0] = 0;
 
 	/* Use old inscription */
-	if (o_ptr->note)
+	if (o_ptr->inscription)
 	{
 		/* Start with the old inscription */
-		strcpy(out_val, quark_str(o_ptr->note));
+		strcpy(out_val, quark_str(o_ptr->inscription));
 	}
 
 	/* Get a new inscription (possibly empty) */
-	if (get_string("Inscription: ", out_val, 80))
+	if (get_string(out_val, 80, "Inscription: "))
 	{
-		/* Save the inscription */
-		o_ptr->note = quark_add(out_val);
+        /* Save the inscription */
+        quark_remove(&o_ptr->inscription);
+		o_ptr->inscription = quark_add(out_val);
 
 		/* Combine the pack */
 		p_ptr->notice |= (PN_COMBINE);
 
 		/* Window stuff */
 		p_ptr->window |= (PW_INVEN | PW_EQUIP);
+
+		make_noise(2);
 	}
 }
 
@@ -801,14 +628,14 @@ void do_cmd_inscribe(void)
 /*
  * An "item_tester_hook" for refilling lanterns
  */
-static bool item_tester_refill_lantern(object_type *o_ptr)
+static bool item_tester_refill_lantern(const object_type *o_ptr)
 {
 	/* Flasks of oil are okay */
 	if (o_ptr->tval == TV_FLASK) return (TRUE);
 
 	/* Laterns are okay */
 	if ((o_ptr->tval == TV_LITE) &&
-	    (o_ptr->sval == SV_LITE_LANTERN)) return (TRUE);
+		(o_ptr->sval == SV_LITE_LANTERN) && (o_ptr->timeout > 0)) return (TRUE);
 
 	/* Assume not okay */
 	return (FALSE);
@@ -820,8 +647,6 @@ static bool item_tester_refill_lantern(object_type *o_ptr)
  */
 static void do_cmd_refill_lamp(void)
 {
-	int item;
-
 	object_type *o_ptr;
 	object_type *j_ptr;
 
@@ -832,71 +657,73 @@ static void do_cmd_refill_lamp(void)
 	item_tester_hook = item_tester_refill_lantern;
 
 	/* Get an item */
-	q = "Refill with which flask? ";
-	s = "You have no flasks of oil.";
-	if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
+	q = "Refill with which source of oil? ";
+	s = "You have no sources of oil.";
 
-	/* Get the item (in the pack) */
-	if (item >= 0)
-	{
-		o_ptr = &inventory[item];
-	}
+	o_ptr = get_item(q, s, (USE_INVEN | USE_FLOOR));
 
-	/* Get the item (on the floor) */
-	else
-	{
-		o_ptr = &o_list[0 - item];
-	}
+	/* Not a valid item */
+	if (!o_ptr) return;
 
 
 	/* Take a partial turn */
-	energy_use = 50;
+	p_ptr->energy_use = 50;
 
 	/* Access the lantern */
-	j_ptr = &inventory[INVEN_LITE];
+	j_ptr = &p_ptr->equipment[EQUIP_LITE];
 
 	/* Refuel */
-	j_ptr->pval += o_ptr->pval;
-
-	/* Message */
-	msg_print("You fuel your lamp.");
-
-	/* Comment */
-	if (j_ptr->pval >= FUEL_LAMP)
+	if (o_ptr->tval == TV_FLASK)
 	{
-		j_ptr->pval = FUEL_LAMP;
-		msg_print("Your lamp is full.");
+		/* Flasks use the pval to store the fuel */
+		j_ptr->timeout += o_ptr->pval;
 	}
-
-	/* Decrease the item (from the pack) */
-	if (item >= 0)
-	{
-		inven_item_increase(item, -1);
-		inven_item_describe(item);
-		inven_item_optimize(item);
-	}
-
-	/* Decrease the item (from the floor) */
 	else
 	{
-		floor_item_increase(0 - item, -1);
-		floor_item_describe(0 - item);
-		floor_item_optimize(0 - item);
+		/* Lanterns use the timeout to store the fuel */
+		j_ptr->timeout += o_ptr->timeout;
 	}
 
+	/* Message */
+	msgf("You fuel your lamp.");
+
+	/* Comment */
+	if (j_ptr->timeout >= FUEL_LAMP)
+	{
+		j_ptr->timeout = FUEL_LAMP;
+		msgf("Your lamp is full.");
+	}
+
+	/* Decrease the item */
+	if (o_ptr->tval == TV_FLASK)
+	{
+		item_increase(o_ptr, -1);
+	}
+	else
+	{
+		/* The lantern is empty */
+		o_ptr->timeout = 0;
+	}
+
+	/* Combine the pack */
+	p_ptr->notice |= (PN_COMBINE | PN_REORDER);
+
 	/* Recalculate torch */
 	p_ptr->update |= (PU_TORCH);
+
+	/* Window stuff */
+	p_ptr->window |= (PW_INVEN | PW_EQUIP);
 }
 
 
 /*
  * An "item_tester_hook" for refilling torches
  */
-static bool item_tester_refill_torch(object_type *o_ptr)
+static bool item_tester_refill_torch(const object_type *o_ptr)
 {
 	/* Torches are okay */
 	if ((o_ptr->tval == TV_LITE) &&
-	    (o_ptr->sval == SV_LITE_TORCH)) return (TRUE);
+		(o_ptr->sval == SV_LITE_TORCH)) return (TRUE);
 
 	/* Assume not okay */
 	return (FALSE);
@@ -908,8 +735,6 @@ static bool item_tester_refill_torch(object_type *o_ptr)
  */
 static void do_cmd_refill_torch(void)
 {
-	int item;
-
 	object_type *o_ptr;
 	object_type *j_ptr;
 
@@ -922,61 +747,39 @@ static void do_cmd_refill_torch(void)
 	/* Get an item */
 	q = "Refuel with which torch? ";
 	s = "You have no extra torches.";
-	if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
 
-	/* Get the item (in the pack) */
-	if (item >= 0)
-	{
-		o_ptr = &inventory[item];
-	}
-
-	/* Get the item (on the floor) */
-	else
-	{
-		o_ptr = &o_list[0 - item];
-	}
+	o_ptr = get_item(q, s, (USE_INVEN | USE_FLOOR));
 
+	/* Not a valid item */
+	if (!o_ptr) return;
 
 	/* Take a partial turn */
-	energy_use = 50;
+	p_ptr->energy_use = 50;
 
 	/* Access the primary torch */
-	j_ptr = &inventory[INVEN_LITE];
+	j_ptr = &p_ptr->equipment[EQUIP_LITE];
 
 	/* Refuel */
-	j_ptr->pval += o_ptr->pval + 5;
+	j_ptr->timeout += o_ptr->timeout + 5;
 
 	/* Message */
-	msg_print("You combine the torches.");
+	msgf("You combine the torches.");
 
 	/* Over-fuel message */
-	if (j_ptr->pval >= FUEL_TORCH)
+	if (j_ptr->timeout >= FUEL_TORCH)
 	{
-		j_ptr->pval = FUEL_TORCH;
-		msg_print("Your torch is fully fueled.");
+		j_ptr->timeout = FUEL_TORCH;
+		msgf("Your torch is fully fueled.");
 	}
 
 	/* Refuel message */
 	else
 	{
-		msg_print("Your torch glows more brightly.");
+		msgf("Your torch glows more brightly.");
 	}
 
-	/* Decrease the item (from the pack) */
-	if (item >= 0)
-	{
-		inven_item_increase(item, -1);
-		inven_item_describe(item);
-		inven_item_optimize(item);
-	}
-
-	/* Decrease the item (from the floor) */
-	else
-	{
-		floor_item_increase(0 - item, -1);
-		floor_item_describe(0 - item);
-		floor_item_optimize(0 - item);
-	}
+	/* Decrease the item */
+	item_increase(o_ptr, -1);
 
 	/* Recalculate torch */
 	p_ptr->update |= (PU_TORCH);
@@ -991,12 +794,12 @@ void do_cmd_refill(void)
 	object_type *o_ptr;
 
 	/* Get the light */
-	o_ptr = &inventory[INVEN_LITE];
+	o_ptr = &p_ptr->equipment[EQUIP_LITE];
 
 	/* It is nothing */
 	if (o_ptr->tval != TV_LITE)
 	{
-		msg_print("You are not wielding a light.");
+		msgf("You are not wielding a light.");
 	}
 
 	/* It's a lamp */
@@ -1014,7 +817,7 @@ void do_cmd_refill(void)
 	/* No torch to refill */
 	else
 	{
-		msg_print("Your light cannot be refilled.");
+		msgf("Your light cannot be refilled.");
 	}
 }
 
@@ -1027,13 +830,13 @@ void do_cmd_target(void)
 	/* Target set */
 	if (target_set(TARGET_KILL))
 	{
-		msg_print("Target Selected.");
+		msgf("Target Selected.");
 	}
 
 	/* Target aborted */
 	else
 	{
-		msg_print("Target Aborted.");
+		msgf("Target Aborted.");
 	}
 }
 
@@ -1047,7 +850,7 @@ void do_cmd_look(void)
 	/* Look around */
 	if (target_set(TARGET_LOOK))
 	{
-		msg_print("Target Selected.");
+		msgf("Target Selected.");
 	}
 }
 
@@ -1058,12 +861,18 @@ void do_cmd_look(void)
  */
 void do_cmd_locate(void)
 {
-	int		dir, y1, x1, y2, x2;
+	int dir, y1, x1, y2, x2;
+	int wid, hgt;
+	char tmp_val[80];
+	char out_val[160];
 
-	char	tmp_val[80];
 
-	char	out_val[160];
+	/* Get size */
+	Term_get_size(&wid, &hgt);
 
+	/* Offset */
+	wid -= COL_MAP + 1;
+	hgt -= ROW_MAP + 1;
 
 	/* Start at current panel */
 	y2 = y1 = panel_row_min;
@@ -1079,16 +888,16 @@ void do_cmd_locate(void)
 		}
 		else
 		{
-			sprintf(tmp_val, "%s%s of",
-			        ((y2 < y1) ? " North" : (y2 > y1) ? " South" : ""),
-			        ((x2 < x1) ? " West" : (x2 > x1) ? " East" : ""));
+			strnfmt(tmp_val, 80, "%s%s of",
+					((y2 < y1) ? " North" : (y2 > y1) ? " South" : ""),
+					((x2 < x1) ? " West" : (x2 > x1) ? " East" : ""));
 		}
 
 		/* Prepare to ask which way to look */
-		sprintf(out_val,
-		        "Map sector [%d(%02d),%d(%02d)], which is%s your sector.  Direction?",
-		        y2 / (SCREEN_HGT / 2), y2 % (SCREEN_HGT / 2),
-		        x2 / (SCREEN_WID / 2), x2 % (SCREEN_WID / 2), tmp_val);
+		strnfmt(out_val, 160,
+				"Map sector [%d(%02d),%d(%02d)], which is%s your sector.  Direction?",
+				y2 / (hgt / 2), y2 % (hgt / 2),
+				x2 / (wid / 2), x2 % (wid / 2), tmp_val);
 
 		/* Assume no direction */
 		dir = 0;
@@ -1105,14 +914,14 @@ void do_cmd_locate(void)
 			dir = get_keymap_dir(command);
 
 			/* Error */
-			if (!dir) bell();
+			if (!dir) bell("Illegal direction for locate!");
 		}
 
 		/* No direction */
 		if (!dir) break;
 
 		/* Apply the motion */
-		if (change_panel(ddy[dir], ddx[dir]))
+		if (change_panel(ddx[dir], ddy[dir]))
 		{
 			y2 = panel_row_min;
 			x2 = panel_col_min;
@@ -1149,8 +958,8 @@ static cptr ident_info[] =
 	"\":An amulet (or necklace)",
 	"#:A wall (or secret door)",
 	"$:Treasure (gold or gems)",
-	"%:A vein (magma or quartz)",
-	/* "&:unused", */
+	"%:Trees",
+	"&:A chest",
 	"':An open door",
 	"(:Soft armor",
 	"):A shield",
@@ -1160,7 +969,7 @@ static cptr ident_info[] =
 	"-:A wand (or rod)",
 	".:Floor",
 	"/:A polearm (Axe/Pike/etc)",
-	/* "0:unused", */
+	/* "0:unused", XXX XXX XXX out of date */
 	"1:Entrance to General Store",
 	"2:Entrance to Armory",
 	"3:Entrance to Weaponsmith",
@@ -1170,8 +979,8 @@ static cptr ident_info[] =
 	"7:Entrance to Black Market",
 	"8:Entrance to your home",
 	"9:Entrance to the bookstore",
-	"::Rubble",
-	";:A glyph of warding / explosive rune",
+	"::Rubble / Rock",
+	";:Swamp / Rune",
 	"<:An up staircase",
 	"=:A ring",
 	">:A down staircase",
@@ -1208,7 +1017,7 @@ static cptr ident_info[] =
 	"]:Misc. armor",
 	"^:A trap",
 	"_:A staff",
-	/* "`:unused", */
+	"`:A figurine or statue",
 	"a:Ant",
 	"b:Bat",
 	"c:Centipede",
@@ -1220,7 +1029,7 @@ static cptr ident_info[] =
 	"i:Icky Thing",
 	"j:Jelly",
 	"k:Kobold",
-	"l:Louse",
+	"l:Aquatic monster",
 	"m:Mold",
 	"n:Naga",
 	"o:Orc",
@@ -1238,7 +1047,7 @@ static cptr ident_info[] =
 	"{:A missile (arrow/bolt/shot)",
 	"|:An edged weapon (sword/dagger/etc)",
 	"}:A launcher (bow/crossbow/sling)",
-	"~:Aquatic monster, tool (or miscellaneous item)",
+	"~:Fluid terrain (or miscellaneous item)",
 	NULL
 };
 
@@ -1250,11 +1059,11 @@ static cptr ident_info[] =
  * We use "u" to point to array of monster indexes,
  * and "v" to select the type of sorting to perform on "u".
  */
-static bool ang_sort_comp_hook(vptr u, vptr v, int a, int b)
+bool ang_sort_comp_hook(const vptr u, const vptr v, int a, int b)
 {
-	u16b *who = (u16b*)(u);
+	u16b *who = (u16b *)(u);
 
-	u16b *why = (u16b*)(v);
+	u16b *why = (u16b *)(v);
 
 	int w1 = who[a];
 	int w2 = who[b];
@@ -1325,12 +1134,15 @@ static bool ang_sort_comp_hook(vptr u, vptr v, int a, int b)
  * We use "u" to point to array of monster indexes,
  * and "v" to select the type of sorting to perform.
  */
-static void ang_sort_swap_hook(vptr u, vptr v, int a, int b)
+void ang_sort_swap_hook(const vptr u, const vptr v, int a, int b)
 {
-	u16b *who = (u16b*)(u);
+	u16b *who = (u16b *)(u);
 
 	u16b holder;
 
+	/* Hack - ignore v */
+	(void)v;
+
 	/* Swap */
 	holder = who[a];
 	who[a] = who[b];
@@ -1338,58 +1150,6 @@ static void ang_sort_swap_hook(vptr u, vptr v, int a, int b)
 }
 
 
-
-/*
- * Hack -- Display the "name" and "attr/chars" of a monster race
- */
-static void roff_top(int r_idx)
-{
-	monster_race	*r_ptr = &r_info[r_idx];
-
-	byte		a1, a2;
-	char		c1, c2;
-
-
-	/* Access the chars */
-	c1 = r_ptr->d_char;
-	c2 = r_ptr->x_char;
-
-	/* Access the attrs */
-	a1 = r_ptr->d_attr;
-	a2 = r_ptr->x_attr;
-
-	/* Hack -- fake monochrome */
-	if (!use_color) a1 = TERM_WHITE;
-	if (!use_color) a2 = TERM_WHITE;
-
-
-	/* Clear the top line */
-	Term_erase(0, 0, 255);
-
-	/* Reset the cursor */
-	Term_gotoxy(0, 0);
-
-	/* A title (use "The" for non-uniques) */
-	if (!(r_ptr->flags1 & (RF1_UNIQUE)))
-	{
-		Term_addstr(-1, TERM_WHITE, "The ");
-	}
-
-	/* Dump the name */
-	Term_addstr(-1, TERM_WHITE, (r_name + r_ptr->name));
-
-	/* Append the "standard" attr/char info */
-	Term_addstr(-1, TERM_WHITE, " ('");
-	Term_addch(a1, c1);
-	Term_addstr(-1, TERM_WHITE, "')");
-
-	/* Append the "optional" attr/char info */
-	Term_addstr(-1, TERM_WHITE, "/('");
-	Term_addch(a2, c2);
-	Term_addstr(-1, TERM_WHITE, "'):");
-}
-
-
 /*
  * Identify a character, allow recall of monsters
  *
@@ -1397,6 +1157,7 @@ static void roff_top(int r_idx)
  *   ^A (all monsters)
  *   ^U (all unique monsters)
  *   ^N (all non-unique monsters)
+ *   ^M (case insensitive name search)
  *
  * The responses may be sorted in several ways, see below.
  *
@@ -1404,24 +1165,26 @@ static void roff_top(int r_idx)
  */
 void do_cmd_query_symbol(void)
 {
-	int		i, n, r_idx;
-	char	sym, query;
-	char	buf[128];
+	int i, n, r_idx;
+	char sym, query;
 
-	bool	all = FALSE;
-	bool	uniq = FALSE;
-	bool	norm = FALSE;
+	bool all = FALSE;
+	bool uniq = FALSE;
+	bool norm = FALSE;
+	bool killed = FALSE;
 
-	bool	recall = FALSE;
+	bool recall = FALSE;
 
-	u16b	why = 0;
-	u16b	*who;
+	u16b why = 0;
+	u16b *who;
 
-	/* Allocate the "who" array */
-	C_MAKE(who, max_r_idx, u16b);
+	char temp1[80] = "\0";
+	char temp2[80] = "\0";
 
 	/* Get a character, or abort */
-	if (!get_com("Enter character to be identified: ", &sym)) return;
+	if (!get_com
+		("Enter character to be identified, or Ctrl (A, U, N, M, K):",
+		 &sym)) return;
 
 	/* Find that character info, and describe it */
 	for (i = 0; ident_info[i]; ++i)
@@ -1433,33 +1196,61 @@ void do_cmd_query_symbol(void)
 	if (sym == KTRL('A'))
 	{
 		all = TRUE;
-		strcpy(buf, "Full monster list.");
+		prtf(0, 0, "Full monster list.");
 	}
 	else if (sym == KTRL('U'))
 	{
 		all = uniq = TRUE;
-		strcpy(buf, "Unique monster list.");
+		prtf(0, 0, "Unique monster list.");
 	}
 	else if (sym == KTRL('N'))
 	{
 		all = norm = TRUE;
-		strcpy(buf, "Non-unique monster list.");
+		prtf(0, 0, "Non-unique monster list.");
 	}
+	else if (sym == KTRL('M'))
+	{
+		all = TRUE;
+		if (!get_string(temp1, 70, "Name:"))
+		{
+			clear_msg();
+		}
+		else
+		{
+			prtf(0, 0, "Monsters with a name \"%s\"", temp1);
+		}
+	}
+	else if (sym == KTRL('K'))
+	{
+		all = killed = TRUE;
+		prtf(0, 0, "Killed monster list.");
+	}
+
 	else if (ident_info[i])
 	{
-		sprintf(buf, "%c - %s.", sym, ident_info[i] + 2);
+		if (ident_info[i][0] == '$')
+		{
+			/*
+			 * Hack - we need two dollar signs since it
+			 * is an escape code.
+			 */
+			prtf(0, 0, "$$ - %s.", ident_info[i] + 2);
+		}
+		else
+		{
+			prtf(0, 0, "%c - %s.", sym, ident_info[i] + 2);
+		}
 	}
 	else
 	{
-		sprintf(buf, "%c - %s.", sym, "Unknown Symbol");
+		prtf(0, 0, "%c - %s.", sym, "Unknown Symbol");
 	}
 
-	/* Display the result */
-	prt(buf, 0, 0);
-
+	/* Allocate the "who" array */
+	C_MAKE(who, z_info->r_max, u16b);
 
 	/* Collect matching monsters */
-	for (n = 0, i = 1; i < max_r_idx; i++)
+	for (n = 0, i = 1; i < z_info->r_max; i++)
 	{
 		monster_race *r_ptr = &r_info[i];
 
@@ -1472,23 +1263,58 @@ void do_cmd_query_symbol(void)
 		/* Require unique monsters if needed */
 		if (uniq && !(r_ptr->flags1 & (RF1_UNIQUE))) continue;
 
+		/* Require killed monsters if needed */
+		if (killed && !r_ptr->r_pkills) continue;
+
+		/* Collect monsters with a name temp1 */
+		if (temp1[0])
+		{
+			int xx;
+
+			for (xx = 0; temp1[xx] && (xx < 80); xx++)
+			{
+				if (isupper(temp1[xx])) temp1[xx] = tolower(temp1[xx]);
+			}
+			strcpy(temp2, r_name + r_ptr->name);
+
+			for (xx = 0; temp2[xx] && (xx < 80); xx++)
+			{
+				if (isupper(temp2[xx])) temp2[xx] = tolower(temp2[xx]);
+			}
+
+			if (strstr(temp2, temp1)) who[n++] = i;
+		}
+
 		/* Collect "appropriate" monsters */
-		if (all || (r_ptr->d_char == sym)) who[n++] = i;
+		else if (all || (r_ptr->d_char == sym)) who[n++] = i;
 	}
 
 	/* Nothing to recall */
-	if (!n) return;
+	if (!n)
+	{
+		/* XXX XXX Free the "who" array */
+		KILL(who);
 
+		return;
+	}
 
 	/* Prompt XXX XXX XXX */
-	put_str("Recall details? (k/p/y/n): ", 0, 40);
+	put_fstr(40, 0, "Recall details? (k/y/n): ");
 
 	/* Query */
 	query = inkey();
+	
+	/* Clear top line */
+	clear_msg();
 
-	/* Restore */
-	prt(buf, 0, 0);
+	why = 2;
 
+	/* Select the sort method */
+	ang_sort_comp = ang_sort_comp_hook;
+	ang_sort_swap = ang_sort_swap_hook;
+
+	/* Sort the array */
+	ang_sort(who, &why, n);
 
 	/* Sort by kills (and level) */
 	if (query == 'k')
@@ -1497,19 +1323,18 @@ void do_cmd_query_symbol(void)
 		query = 'y';
 	}
 
-	/* Sort by level */
-	if (query == 'p')
-	{
-		why = 2;
-		query = 'y';
-	}
-
 	/* Catch "escape" */
-	if (query != 'y') return;
+	if (query != 'y')
+	{
+		/* XXX XXX Free the "who" array */
+		KILL(who);
+
+		return;
+	}
 
 
 	/* Sort if needed */
-	if (why)
+	if (why == 4)
 	{
 		/* Select the sort method */
 		ang_sort_comp = ang_sort_comp_hook;
@@ -1539,7 +1364,7 @@ void do_cmd_query_symbol(void)
 		roff_top(r_idx);
 
 		/* Hack -- Complete the prompt */
-		Term_addstr(-1, TERM_WHITE, " [(r)ecall, ESC]");
+		roff(" [(r)ecall, ESC]");
 
 		/* Interact */
 		while (1)
@@ -1554,7 +1379,7 @@ void do_cmd_query_symbol(void)
 				screen_roff(who[i], 0);
 
 				/* Hack -- Complete the prompt (again) */
-				Term_addstr(-1, TERM_WHITE, " [(r)ecall, ESC]");
+				roff(" [(r)ecall, ESC]");
 			}
 
 			/* Command */
@@ -1598,8 +1423,11 @@ void do_cmd_query_symbol(void)
 		}
 	}
 
-	/* Re-display the identity */
-	prt(buf, 0, 0);
+	/* Free the "who" array */
+	KILL(who);
+	
+	/* Clear top line */
+	clear_msg();
 }
 
 
@@ -1611,7 +1439,6 @@ bool research_mon(void)
 {
 	int i, n, r_idx;
 	char sym, query;
-	char buf[128];
 
 	s16b oldkills;
 	byte oldwake;
@@ -1625,7 +1452,7 @@ bool research_mon(void)
 
 	monster_race *r2_ptr;
 
-	u16b	*who;
+	u16b *who;
 
 	oldcheat = cheat_know;
 
@@ -1643,7 +1470,7 @@ bool research_mon(void)
 	}
 
 	/* Allocate the "who" array */
-	C_MAKE(who, max_r_idx, u16b);
+	C_MAKE(who, z_info->r_max, u16b);
 
 	/* Find that character info, and describe it */
 	for (i = 0; ident_info[i]; ++i)
@@ -1653,19 +1480,16 @@ bool research_mon(void)
 
 	if (ident_info[i])
 	{
-		sprintf(buf, "%c - %s.", sym, ident_info[i] + 2);
+		prtf(10, 16, "%c - %s.", sym, ident_info[i] + 2);
 	}
 	else
 	{
-		sprintf(buf, "%c - %s.", sym, "Unknown Symbol");
+		prtf(10, 16, "%c - %s.", sym, "Unknown Symbol");
 	}
 
-	/* Display the result */
-	prt(buf, 16, 10);
-
 
 	/* Collect matching monsters */
-	for (n = 0, i = 1; i < max_r_idx; i++)
+	for (n = 0, i = 1; i < z_info->r_max; i++)
 	{
 		monster_race *r_ptr = &r_info[i];
 
@@ -1681,7 +1505,7 @@ bool research_mon(void)
 		cheat_know = oldcheat;
 
 		/* Free the "who" array */
-		C_KILL(who, max_r_idx, u16b);
+		KILL(who);
 
 		/* Restore */
 		screen_load();
@@ -1726,7 +1550,7 @@ bool research_mon(void)
 		roff_top(r_idx);
 
 		/* Hack -- Complete the prompt */
-		Term_addstr(-1, TERM_WHITE, " [(r)ecall, ESC, space to continue]");
+		roff(" [(r)ecall, ESC, space to continue]");
 
 		/* Interact */
 		while (1)
@@ -1780,18 +1604,13 @@ bool research_mon(void)
 		}
 	}
 
-
-	/* Re-display the identity */
-	/* prt(buf, 5, 5);*/
-
 	cheat_know = oldcheat;
 
 	/* Free the "who" array */
-	C_KILL(who, max_r_idx, u16b);
+	KILL(who);
 
 	/* Restore */
 	screen_load();
 
 	return (!notpicked);
 }
-
