@@ -15,6 +15,8 @@
 /* Current maintainer: silasd@psyber.com (Silas Dunsmore) */
 /* Unless somebody else wants it.... */
 
+#include "angband.h"
+
 #ifdef USE_EMX
 
 /*
@@ -108,8 +110,6 @@
 #include <os2.h>
 #include <sys/video.h>
 
-#include "angband.h"
-
 
 /*
  * Maximum windows
@@ -816,7 +816,7 @@ static FILE *initPipe(const char *name)
 	char buf[256];
 	FILE *fi;
 
-	sprintf(buf, "\\pipe\\angband\\%s", name);   /* Name of pipe */
+	strnfmt(buf, 256, "\\pipe\\angband\\%s", name);   /* Name of pipe */
 	fi=fopen(buf, "wb");                        /* Look for server */
 	return fi;
 }
@@ -851,7 +851,7 @@ int main(int argc, char **argv)
 	target=strdup(argv[1]);
 	for (c=0; c<strlen(target); c++) target[c]=tolower(target[c]);
 
-	sprintf(buf, "\\pipe\\angband\\%s", target);
+	strnfmt(buf, 160, "\\pipe\\angband\\%s", target);
 
 	do
 	{
@@ -892,7 +892,7 @@ int main(int argc, char **argv)
 
 	printf("Connected.\n");
 
-	sprintf(buf, "mode co80,%d", lines);
+	strnfmt(buf, 160, "mode co80,%d", lines);
 	system(buf);
 
 	/* Infinite loop */
