@@ -37,7 +37,7 @@
  * This file will attempt to redefine the screen colors to conform to
  * standard Angband colors.  It will only do so if the terminal type
  * indicates that it can do so.  See the page:
- * 
+ *
  *     http://www.umr.edu/~keldon/ang-patch/ncurses_color.html
  *
  * for information on this.
@@ -51,6 +51,12 @@
 
 #ifdef USE_GCU
 
+cptr help_gcu[] =
+{
+	"To use GCU (GNU Curses)",
+	NULL
+};
+
 /*
  * Hack -- play games with "bool"
  */
@@ -156,6 +162,11 @@
 /* #define nl() */
 
 
+/*
+ * OPTION: NetBSD seems to like this better
+ */
+/* #define can_change_color() can_change_colors() */
+
 /*
  * Save the "normal" and "angband" terminal settings
  */
@@ -240,9 +251,14 @@ static int can_fix_color = FALSE;
  */
 static int colortable[16];
 
-#endif
+#endif /* A_COLOR */
 
 
+#ifdef USE_GRAPHICS
+
+static bool use_blocks = FALSE;
+
+#endif /* USE_GRAPHICS */
 
 /*
  * Place the "keymap" into its "normal" state
@@ -455,6 +471,9 @@ static void keymap_game_prepare(void)
  */
 static errr Term_xtra_gcu_alive(int v)
 {
+	int x, y;
+
+
 	/* Suspend */
 	if (!v)
 	{
@@ -472,13 +491,11 @@ static errr Term_xtra_gcu_alive(int v)
 		/* Flush the curses buffer */
 		(void)refresh();
 
-#ifdef SPECIAL_BSD
-		/* this moves curses to bottom right corner */
-		mvcur(curscr->cury, curscr->curx, LINES - 1, 0);
-#else
-		/* this moves curses to bottom right corner */
-		mvcur(curscr->_cury, curscr->_curx, LINES - 1, 0);
-#endif
+		/* Get current cursor position */
+		getyx(curscr, y, x);
+
+		/* Move the cursor to bottom right corner */
+		mvcur(y, x, LINES - 1, 0);
 
 		/* Exit curses */
 		endwin();
@@ -539,6 +556,7 @@ static void Term_init_gcu(term *t)
  */
 static void Term_nuke_gcu(term *t)
 {
+	int x, y;
 	term_data *td = (term_data *)(t->data);
 
 	/* Delete this window */
@@ -555,13 +573,11 @@ static void Term_nuke_gcu(term *t)
 	start_color();
 #endif
 
-#ifdef SPECIAL_BSD
-	/* This moves curses to bottom right corner */
-	mvcur(curscr->cury, curscr->curx, LINES - 1, 0);
-#else
-	/* This moves curses to bottom right corner */
-	mvcur(curscr->_cury, curscr->_curx, LINES - 1, 0);
-#endif
+	/* Get current cursor position */
+	getyx(curscr, y, x);
+
+	/* Move the cursor to bottom right corner */
+	mvcur(y, x, LINES - 1, 0);
 
 	/* Flush the curses buffer */
 	(void)refresh();
@@ -759,7 +775,7 @@ static errr Term_xtra_gcu(int n, int v)
 
 		/* Delay */
 		case TERM_XTRA_DELAY:
-		usleep(1000 * v);
+		if (v > 0) usleep(1000 * v);
 		return (0);
 
 		/* React to events */
@@ -823,7 +839,10 @@ static errr Term_text_gcu(int x, int y, int n, byte a, cptr s)
 {
 	term_data *td = (term_data *)(Term->data);
 
-	int i, pic;
+	int i;
+#ifdef USE_GRAPHICS
+	int pic;
+#endif
 
 #ifdef A_COLOR
 	/* Set the color */
@@ -837,27 +856,28 @@ static errr Term_text_gcu(int x, int y, int n, byte a, cptr s)
 	for (i = 0; i < n; i++)
 	{
 #ifdef USE_GRAPHICS
-		/* Special character */
-		if (use_graphics && (s[i] & 0x80))
+		/* Special characters? */
+		if (use_blocks)
 		{
+#ifdef ACS_CKBOARD
 			/* Determine picture to use */
-			switch (s[i] & 0x7F)
+			if (s[i] == '#')
 			{
-				/* Wall */
-				case '#':
-					pic = ACS_CKBOARD;
-					break;
-
-				/* Mineral vein */
-				case '%':
-					pic = ACS_BOARD;
-					break;
-
-				/* XXX */
-				default:
-					pic = '?';
-					break;
+				/* Walls */
+				pic = ACS_CKBOARD;
+				
+				/*
+				 *  Note that veins are '#' as well now.
+				 *  Trees are '%' now - and this looks bad when redefined.
+				 */
 			}
+			else
+			{
+				pic = s[i];
+			}
+#else /* ACS_CKBOARD */
+			pic = s[i];
+#endif /* ACS_CKBOARD */
 
 			/* Draw the picture */
 			waddch(td->win, pic);
@@ -878,7 +898,7 @@ static errr Term_text_gcu(int x, int y, int n, byte a, cptr s)
 /*
  * Create a window for the given "term_data" argument.
  */
-static errr term_data_init_gcu(term_data *td, int rows, int cols, int y, int x)
+static errr term_data_init_gcu(term_data *td, int rows, int cols, int y, int x, int i)
 {
 	term *t = &td->t;
 
@@ -921,20 +941,43 @@ static errr term_data_init_gcu(term_data *td, int rows, int cols, int y, int x)
 	/* Activate it */
 	Term_activate(t);
 
+	/* Reset map size if required */
+	if (i == 0)
+	{
+		/* Mega-Hack -- no panel yet */
+		panel_row_min = 0;
+		panel_row_max = 0;
+		panel_col_min = 0;
+		panel_col_max = 0;
+
+		/* Reset the panels */
+		map_panel_size();
+	}
+
 	/* Success */
 	return (0);
 }
 
+/* The hook to exit curses on a failure */
+static void hook_quit(cptr str)
+{
+	/* Ignore parameter */
+	(void) str;
+	
+	/* Exit curses */
+	endwin();
+}
+
 
 /*
- * Prepare "curses" for use by the file "term.c"
+ * Prepare "curses" for use by the file "z-term.c"
  *
  * Installs the "hook" functions defined above, and then activates
  * the main screen "term", which clears the screen and such things.
  *
  * Someone should really check the semantics of "initscr()"
  */
-errr init_gcu(int argc, char *argv[])
+errr init_gcu(void)
 {
 	int i;
 
@@ -948,18 +991,26 @@ errr init_gcu(int argc, char *argv[])
 	/* Initialize for USG Unix */
 	if (initscr() == NULL) return (-1);
 #else
-	/* Initialize for others systems */
+	/* Initialize for other systems */
 	if (initscr() == (WINDOW*)ERR) return (-1);
 #endif
 
+	/* Activate hooks */
+	quit_aux = hook_quit;
+	core_aux = hook_quit;
 
-	/* Hack -- Require large screen, or Quit with message */
-	i = ((LINES < 24) || (COLS < 80));
-	if (i) quit("Angband needs an 80x24 'curses' screen");
+	/* Require standard size screen */
+	if ((LINES < 24) || (COLS < 80))
+	{
+		quit("Angband needs at least an 80x24 'curses' screen");
+	}
 
 #ifdef USE_GRAPHICS
 	/* Set graphics flag */
-	use_graphics = arg_graphics;
+	use_graphics = GRAPHICS_NONE;
+
+	/* Use the graphical wall tiles? */
+	use_blocks = arg_graphics;
 #endif
 
 #ifdef A_COLOR
@@ -1060,38 +1111,48 @@ errr init_gcu(int argc, char *argv[])
 	{
 		int rows, cols, y, x;
 
+		/* Hack - the main window is huge */
+		
+		/* Work out how much extra room we have */
+		cols = (COLS - 80) / 4;
+		rows = (LINES - 24) / 2;
+		
+		/* Prevent stupidly small windows */
+		if (cols < 25) cols = 0;
+		if (rows < 10) rows = 0;
+		
+		/* Ok - so we now have the size of the main window */
+		cols += 80;
+		rows += 24;
+
 		/* Decide on size and position */
 		switch (i)
 		{
 			/* Upper left */
 			case 0:
-				rows = 24;
-				cols = 80;
 				y = x = 0;
 				break;
 
 			/* Lower left */
-			case 1:
-				rows = LINES - 25;
-				cols = 80;
-				y = 25;
+			case 1:				
+				y = rows + 1;
 				x = 0;
+				rows = LINES - (rows + 1);
 				break;
 
 			/* Upper right */
-			case 2:
-				rows = 24;
-				cols = COLS - 81;
+			case 2:				
 				y = 0;
-				x = 81;
+				x = cols + 1;
+				cols = COLS - (cols + 1);
 				break;
 
 			/* Lower right */
-			case 3:
-				rows = LINES - 25;
-				cols = COLS - 81;
-				y = 25;
-				x = 81;
+			case 3:				
+				y = rows + 1;
+				x = cols + 1;
+				rows = LINES - (rows + 1);
+				cols = COLS - (cols + 1);				
 				break;
 
 			/* XXX */
@@ -1104,7 +1165,7 @@ errr init_gcu(int argc, char *argv[])
 		if (rows <= 0 || cols <= 0) continue;
 
 		/* Create a term */
-		term_data_init_gcu(&data[next_win], rows, cols, y, x);
+		term_data_init_gcu(&data[next_win], rows, cols, y, x, i);
 
 		/* Remember the term */
 		angband_term[next_win] = &data[next_win].t;
@@ -1123,7 +1184,5 @@ errr init_gcu(int argc, char *argv[])
 	return (0);
 }
 
-
 #endif /* USE_GCU */
 
-
