@@ -8,7 +8,7 @@
  * are included in all such copies.
  */
 
-/* Purpose: Visual Display Support for "term.c", for the IBM */
+/* Purpose: Visual Display Support for "z-term.c", for the IBM */
 
 
 /*
@@ -54,6 +54,12 @@
 
 #ifdef USE_IBM
 
+cptr help_ibm[] =
+{
+	"To use IBM (BIOS text mode)",
+	NULL
+};
+
 
 /*
  * Use a "virtual" screen to "buffer" screen writes.
@@ -378,7 +384,7 @@ static void activate_color_complex(void)
 
 #endif /* 1 */
 
-};
+}
 
 
 /*
@@ -871,16 +877,15 @@ static errr Term_text_ibm(int x, int y, int n, byte a, const char *cp)
  *
  * The given parameters are "valid".
  */
-#ifdef USE_TRANSPARENCY
 static errr Term_pict_ibm(int x, int y, int n, const byte *ap, const char *cp, const byte *tap, const char *tcp)
-#else /* USE_TRANSPARENCY */
-static errr Term_pict_ibm(int x, int y, int n, const byte *ap, const char *cp)
-#endif /* USE_TRANSPARENCY */
 {
 	register int i;
 	register byte attr;
 	register byte *dest;
 
+	/* Unused parameter */
+	(void)tap;
+	(void)tcp;
 
 #ifdef USE_CONIO
 
@@ -950,6 +955,9 @@ static errr Term_pict_ibm(int x, int y, int n, const byte *ap, const char *cp)
  */
 static void Term_init_ibm(term *t)
 {
+	/* Unused parameter */
+	(void)t;
+
 	/* XXX Nothing */
 }
 
@@ -959,7 +967,6 @@ static void Term_init_ibm(term *t)
  */
 static void Term_nuke_ibm(term *t)
 {
-
 #ifdef USE_WAT
 
 	/* Nothing */
@@ -970,6 +977,9 @@ static void Term_nuke_ibm(term *t)
 
 #endif /* USE_WAT */
 
+	/* Unused parameter */
+	(void)t;
+
 	/* Move the cursor to the bottom of the screen */
 	Term_curs_ibm(0, rows-1);
 
@@ -1140,7 +1150,8 @@ void enable_graphic_font(const char *font)
 {
 	__dpmi_regs dblock = {{0}};
 
-	unsigned seg, sel, i;
+	unsigned int seg, i;
+	int sel;
 
 	/*
 	 * Allocate a block of memory 4096 bytes big in `low memory' so a real
@@ -1176,7 +1187,7 @@ void enable_graphic_font(const char *font)
 
 	/* We're done with the low memory, free it */
 	__dpmi_free_dos_memory(sel);
-};
+}
 
 #endif /* USE_286 */
 
@@ -1310,7 +1321,7 @@ errr init_ibm(void)
 			activate_color_complex();
 
 			/* Use graphics */
-			use_graphics = TRUE;
+			use_graphics = GRAPHICS_ORIGINAL;
 		}
 	}
 
