@@ -19,7 +19,7 @@
 static int highscore_seek(int i)
 {
 	/* Seek for the requested record */
-	return (fd_seek(highscore_fd, (huge)(i) * sizeof(high_score)));
+	return (fd_seek(highscore_fd, (huge) (i) * sizeof(high_score)));
 }
 
 
@@ -29,17 +29,17 @@ static int highscore_seek(int i)
 static errr highscore_read(high_score *score)
 {
 	/* Read the record, note failure */
-	return (fd_read(highscore_fd, (char*)(score), sizeof(high_score)));
+	return (fd_read(highscore_fd, (char *)(score), sizeof(high_score)));
 }
 
 
 /*
  * Write one score to the highscore file
  */
-static int highscore_write(high_score *score)
+static int highscore_write(const high_score *score)
 {
 	/* Write the record, note failure */
-	return (fd_write(highscore_fd, (char*)(score), sizeof(high_score)));
+	return (fd_write(highscore_fd, (char *)(score), sizeof(high_score)));
 }
 
 
@@ -47,11 +47,11 @@ static int highscore_write(high_score *score)
  * Just determine where a new score *would* be placed
  * Return the location (0 is best) or -1 on failure
  */
-static int highscore_where(high_score *score)
+static int highscore_where(const high_score *score)
 {
-	int			i;
+	int i;
 
-	high_score		the_score;
+	high_score the_score;
 
 	/* Paranoia -- it may not have opened */
 	if (highscore_fd < 0) return (-1);
@@ -75,12 +75,12 @@ static int highscore_where(high_score *score)
  * Actually place an entry into the high score file
  * Return the location (0 is best) or -1 on "failure"
  */
-static int highscore_add(high_score *score)
+static int highscore_add(const high_score *score)
 {
-	int			i, slot;
-	bool		done = FALSE;
+	int i, slot;
+	bool done = FALSE;
 
-	high_score		the_score, tmpscore;
+	high_score the_score, tmpscore;
 
 
 	/* Paranoia -- it may not have opened */
@@ -115,6 +115,85 @@ static int highscore_add(high_score *score)
 }
 
 
+/*
+ * How much valuable stuff do we carry. Returns total cost of all found
+ * items currently in equipment (ie. those that haven't been storebought
+ * or started with)
+ */
+static long equip_value(void)
+{
+	object_type *o_ptr;
+	long total = 0L;
+	int i;
+
+	/* Scan equipment */
+	for (i = 0; i < EQUIP_MAX; i++)
+	{
+		o_ptr = &p_ptr->equipment[i];
+
+		if (o_ptr->info & OB_STOREB) continue;
+		if (!(o_ptr->info & OB_KNOWN)) continue;
+
+		total += object_value(o_ptr);
+	}
+
+	/* Scan inventory */
+	OBJ_ITT_START (p_ptr->inventory, o_ptr)
+	{
+		if (o_ptr->info & OB_STOREB) continue;
+		if (!(o_ptr->info & OB_KNOWN)) continue;
+
+		total += object_value(o_ptr);
+	}
+	OBJ_ITT_END;
+
+	return (total);
+}
+
+
+/*
+ * Hack -- Calculates the total number of points earned         -JWT-
+ * Now with up to 80% penalty for having mutations & other extra things  -GSN-
+ * Fixed this up to be "fairer" -CK-
+ */
+static long total_points(void)
+{
+	long temp;
+	long mult = 85;
+
+	/* AI is not that big a deal (yet) */
+	if (stupid_monsters) mult -= 20;
+
+	/* Penalize preserve mode */
+	if (preserve_mode) mult -= 10;
+
+	/* Vanilla town is harder than normal */
+	if (vanilla_town) mult += 5;
+
+	/* so are hard quests */
+	if (ironman_hard_quests) mult += number_of_quests() / 2;
+	if (ironman_deep_quests) mult += number_of_quests();
+
+	/* Not too much of a reward since some people like playing with this. */
+	if (ironman_small_levels) mult += 5;
+
+	/* More ironman options */
+	if (ironman_empty_levels) mult += 10;
+	if (ironman_nightmare) mult += 20;
+
+	if (mult < 5) mult = 5;		/* At least 5% of the original score */
+
+	temp = p_ptr->max_exp + (100 * p_ptr->max_depth);
+
+	temp = (temp * mult) / race_info[p_ptr->rp.prace].r_exp;
+
+	temp += equip_value() / 10;
+
+	if (ironman_downward) temp *= 2;
+
+	return (temp);
+}
+
 
 /*
  * Display the scores in a given range.
@@ -123,16 +202,16 @@ static int highscore_add(high_score *score)
  *
  * Mega-Hack -- allow "fake" entry at the given position.
  */
-void display_scores_aux(int from, int to, int note, high_score *score)
+void display_scores_aux(int from, int to, int note, const high_score *score)
 {
-	int		i, j, k, n, place;
+	int i, j, k, n, place;
 	byte attr;
 
-	high_score	the_score;
-
-	char	out_val[256];
-	char	tmp_val[160];
+	high_score the_score;
 
+	char out_val[256];
+	
+	int len;
 
 	/* Paranoia -- it may not have opened */
 	if (highscore_fd < 0) return;
@@ -161,19 +240,18 @@ void display_scores_aux(int from, int to, int note, high_score *score)
 
 
 	/* Show 5 per page, until "done" */
-	for (k = from, place = k+1; k < i; k += 5)
+	for (k = from, place = k + 1; k < i; k += 5)
 	{
 		/* Clear screen */
 		Term_clear();
 
 		/* Title */
-		put_str("                Zangband Hall of Fame", 0, 0);
+		put_fstr(0, 0, "                %s Hall of Fame", VERSION_NAME);
 
 		/* Indicate non-top scores */
 		if (k > 0)
 		{
-			sprintf(tmp_val, "(from position %d)", k + 1);
-			put_str(tmp_val, 0, 40);
+			put_fstr(40, 0, "(from position %d)", k + 1);
 		}
 
 		/* Dump 5 entries */
@@ -217,59 +295,55 @@ void display_scores_aux(int from, int to, int note, high_score *score)
 			mdun = atoi(the_score.max_dun);
 
 			/* Hack -- extract the gold and such */
-			for (user = the_score.uid; isspace(*user); user++) /* loop */;
-			for (when = the_score.day; isspace(*when); when++) /* loop */;
-			for (gold = the_score.gold; isspace(*gold); gold++) /* loop */;
-			for (aged = the_score.turns; isspace(*aged); aged++) /* loop */;
+			for (user = the_score.uid; isspace(*user); user++) /* loop */ ;
+			for (when = the_score.day; isspace(*when); when++) /* loop */ ;
+			for (gold = the_score.gold; isspace(*gold); gold++) /* loop */ ;
+			for (aged = the_score.turns; isspace(*aged); aged++) /* loop */ ;
 
 			/* Dump some info */
-			sprintf(out_val, "%3d.%9s  %s the %s %s, Level %d",
-			        place, the_score.pts, the_score.who,
-			        race_info[pr].title, class_info[pc].title,
-			        clev);
+			len = strnfmt(out_val, 256, "%3d.%9s  %s the %s %s, Level %d",
+					place, the_score.pts, the_score.who,
+					race_info[pr].title, class_info[pc].title, clev);
 
 			/* Append a "maximum level" */
-			if (mlev > clev) strcat(out_val, format(" (Max %d)", mlev));
+			if (mlev > clev) strnfcat(out_val, 256, &len, " (Max %d)", mlev);
 
 			/* Dump the first line */
-			c_put_str(attr, out_val, n*4 + 2, 0);
-
-			/* Another line of info */
-			sprintf(out_val, "               Killed by %s on %s %d",
-			        the_score.how, "Dungeon Level", cdun);
-
+			put_fstr(0, n * 4 + 2, "%s%s", color_seq[attr], out_val);
 
 			/* Some people die outside of the dungeon */
-			if (!cdun)
+			if (cdun)
 			{
-				/* Died in town */
-				if (p_ptr->town_num)
-					sprintf(out_val, "               Killed by %s in the Town",
-					        the_score.how);
-				/* Died in the wilderness */
-				else
-					sprintf(out_val, "               Killed by %s in the Wilderness",
-					        the_score.how);
+				/* Another line of info */
+				len = strnfmt(out_val, 256, "               Killed by %s on %s %d",
+						the_score.how, "Dungeon Level", cdun);
+
+			}
+			else
+			{
+				/* Died outside */
+				len = strnfmt(out_val, 256,
+						"               Killed by %s in the outside world.",
+						the_score.how);
 			}
 
 			/* Append a "maximum level" */
-			if (mdun > cdun) strcat(out_val, format(" (Max %d)", mdun));
+			if (mdun > cdun) strnfcat(out_val, 256, &len, " (Max %d)", mdun);
 
 			/* Dump the info */
-			c_put_str(attr, out_val, n*4 + 3, 0);
+			put_fstr(0, n * 4 + 3, "%s%s", color_seq[attr], out_val);
 
 			/* And still another line of info */
-			sprintf(out_val,
-			        "               (User %s, Date %s, Gold %s, Turn %s).",
-			        user, when, gold, aged);
-			c_put_str(attr, out_val, n*4 + 4, 0);
+			put_fstr(0, n * 4 + 4,
+				"%s               (User %s, Date %s, Gold %s, Turn %s).",
+					 color_seq[attr], user, when, gold, aged);
 		}
 
 
 		/* Wait for response */
-		prt("[Press ESC to quit, any other key to continue.]", 23, 17);
+		prtf(17, 23, "[Press ESC to quit, any other key to continue.]");
 		j = inkey();
-		prt("", 23, 0);
+		clear_row(23);
 
 		/* Hack -- notice Escape */
 		if (j == ESCAPE) break;
@@ -313,209 +387,255 @@ void display_scores(int from, int to)
 }
 
 
+/*
+ * Hack - save index of player's high score
+ */
+static int score_idx = -1;
+
 
 /*
- * Enters a players name on a hi-score table, if "legal", and in any
- * case, displays some relevant portion of the high score list.
+ * Enters a players name on a hi-score table, if "legal".
  *
  * Assumes "signals_ignore_tstp()" has been called.
  */
-errr top_twenty(void)
+void enter_score(void)
 {
-	int          j;
+	int i;
+	high_score the_score;
 
-	high_score   the_score;
+#ifndef HIGHSCORE_DATE_HACK
+	char long_day[12];
+#endif
 
-	time_t ct = time((time_t*)0);
+	time_t ct = time((time_t *) 0);
+
+	/* No score file */
+	if (highscore_fd < 0)
+	{
+		return;
+	}
+
+#ifndef SCORE_WIZARDS
+	/* Wizard-mode pre-empts scoring */
+	if (p_ptr->state.noscore & 0x003F)
+	{
+		msgf("Score not registered for wizards.");
+		message_flush();
+		score_idx = -1;
+		return;
+	}
+#endif
+
+#ifndef SCORE_BORGS
+	/* Borg-mode pre-empts scoring */
+	if (p_ptr->state.noscore & 0x00C0)
+	{
+		msgf("Score not registered for borgs.");
+		message_flush();
+		score_idx = -1;
+		return;
+	}
+#endif
+
+#ifndef SCORE_CHEATERS
+	/* Cheaters are not scored */
+	if (p_ptr->state.noscore & 0xFF00)
+	{
+		msgf("Score not registered for cheaters.");
+		message_flush();
+		score_idx = -1;
+		return;
+	}
+#endif
+
+	/* Interupted */
+	if (!p_ptr->state.total_winner && streq(p_ptr->state.died_from, "Interrupting"))
+	{
+		msgf("Score not registered due to interruption.");
+		message_flush();
+		score_idx = -1;
+		return;
+	}
+
+	/* Quitter */
+	if (!p_ptr->state.total_winner && streq(p_ptr->state.died_from, "Quitting"))
+	{
+		msgf("Score not registered due to quitting.");
+		message_flush();
+		score_idx = -1;
+		return;
+	}
 
 
+	/* Clear the record */
+	(void)WIPE(&the_score, high_score);
+
+	/* Save the version */
+	strnfmt(the_score.what, 8, "%u.%u.%u",
+			VER_MAJOR, VER_MINOR, VER_PATCH);
+
+	/* Calculate and save the points */
+	strnfmt(the_score.pts, 10, "%9lu", (unsigned long)total_points());
+	the_score.pts[9] = '\0';
+
+	/* Save the current gold */
+	strnfmt(the_score.gold, 10, "%9lu", (unsigned long)p_ptr->au);
+	the_score.gold[9] = '\0';
+
+	/* Save the current turn */
+	strnfmt(the_score.turns, 10, "%9lu", (unsigned long)turn);
+	the_score.turns[9] = '\0';
+
+#ifdef HIGHSCORE_DATE_HACK
+	/* Save the date in a hacked up form (9 chars) */
+	(void)strnfmt(the_score.day, 10, "%-.6s %-.2s", ctime(&ct) + 4,
+				  ctime(&ct) + 22);
+#else  /* HIGHSCORE_DATE_HACK */
+	/* Get the date with a 4-digit year */
+	(void)strftime(long_day, 11, "%m/%d/%Y", localtime(&ct));
+
+	/* Remove the century */
+	i = 7;
+	while (1)
+	{
+		i++;
+		long_day[i - 2] = long_day[i];
+
+		/* Exit if get a zero */
+		if (!long_day[i] || (i == 11)) break;
+	}
+
+	/* Save the date in standard form (8 chars) */
+	(void)strnfmt(the_score.day, 9, "%s", long_day);
+#endif /* HIGHSCORE_DATE_HACK */
+
+	/* Save the player name (15 chars) */
+	strnfmt(the_score.who, 16, "%-.15s", player_name);
+
+	/* Save the player info XXX XXX XXX */
+	strnfmt(the_score.uid, 8, "%7u", (uint)player_uid);
+	strnfmt(the_score.sex, 2, "%c", (p_ptr->rp.psex ? 'm' : 'f'));
+	strnfmt(the_score.p_r, 3, "%2d", (int)p_ptr->rp.prace);
+	strnfmt(the_score.p_c, 3, "%2d", (int)p_ptr->rp.pclass);
+
+	/* Save the level and such */
+	strnfmt(the_score.cur_lev, 4, "%3d", p_ptr->lev);
+	strnfmt(the_score.cur_dun, 4, "%3d", p_ptr->depth);
+	strnfmt(the_score.max_lev, 4, "%3d", p_ptr->max_lev);
+	strnfmt(the_score.max_dun, 4, "%3d", p_ptr->max_depth);
+
+	/* Save the cause of death (31 chars) */
+	strnfmt(the_score.how, 32, "%-.31s", p_ptr->state.died_from);
+
+	/* Grab permissions */
+	safe_setuid_grab();
+
+	/* Lock (for writing) the highscore file, or fail */
+	if (fd_lock(highscore_fd, F_WRLCK)) return;
+
+	/* Drop permissions */
+	safe_setuid_drop();
+
+	/* Add a new entry to the score list, see where it went */
+	score_idx = highscore_add(&the_score);
+
+	/* Grab permissions */
+	safe_setuid_grab();
+
+	/* Unlock the highscore file, or fail */
+	if (fd_lock(highscore_fd, F_UNLCK)) return;
+
+	/* Drop permissions */
+	safe_setuid_drop();
+
+	/* Success */
+	return;
+}
+
+
+
+/*
+ * Displays some relevant portion of the high score list.
+ */
+void top_twenty(void)
+{
 	/* Clear screen */
 	Term_clear();
 
 	/* No score file */
 	if (highscore_fd < 0)
 	{
-		msg_print("Score file unavailable.");
-		msg_print(NULL);
-		return (0);
+		msgf("Score file unavailable.");
+		message_flush();
+		return;
 	}
 
-#ifndef SCORE_WIZARDS
-	/* Wizard-mode pre-empts scoring */
-	if (noscore & 0x000F)
-	{
-		msg_print("Score not registered for wizards.");
-		msg_print(NULL);
-		display_scores_aux(0, 10, -1, NULL);
-		return (0);
-	}
-#endif
-
-#ifndef SCORE_BORGS
-	/* Borg-mode pre-empts scoring */
-	if (noscore & 0x00F0)
-	{
-		msg_print("Score not registered for borgs.");
-		msg_print(NULL);
-		display_scores_aux(0, 10, -1, NULL);
-		return (0);
-	}
-#endif
-
-#ifndef SCORE_CHEATERS
-	/* Cheaters are not scored */
-	if (noscore & 0xFF00)
-	{
-		msg_print("Score not registered for cheaters.");
-		msg_print(NULL);
-		display_scores_aux(0, 10, -1, NULL);
-		return (0);
-	}
-#endif
-
-	/* Interupted */
-	if (!total_winner && streq(died_from, "Interrupting"))
-	{
-		msg_print("Score not registered due to interruption.");
-		msg_print(NULL);
-		display_scores_aux(0, 10, -1, NULL);
-		return (0);
-	}
-
-	/* Quitter */
-	if (!total_winner && streq(died_from, "Quitting"))
-	{
-		msg_print("Score not registered due to quitting.");
-		msg_print(NULL);
-		display_scores_aux(0, 10, -1, NULL);
-		return (0);
-	}
-
-
-	/* Clear the record */
-	WIPE(&the_score, high_score);
-
-	/* Save the version */
-	sprintf(the_score.what, "%u.%u.%u",
-	        VERSION_MAJOR, VERSION_MINOR, VERSION_PATCH);
-
-	/* Calculate and save the points */
-	sprintf(the_score.pts, "%9lu", (long)total_points());
-	the_score.pts[9] = '\0';
-
-	/* Save the current gold */
-	sprintf(the_score.gold, "%9lu", (long)p_ptr->au);
-	the_score.gold[9] = '\0';
-
-	/* Save the current turn */
-	sprintf(the_score.turns, "%9lu", (long)turn);
-	the_score.turns[9] = '\0';
-
-#ifdef HIGHSCORE_DATE_HACK
-	/* Save the date in a hacked up form (9 chars) */
-	(void)sprintf(the_score.day, "%-.6s %-.2s", ctime(&ct) + 4, ctime(&ct) + 22);
-#else
-	/* Save the date in standard form (8 chars) */
-	(void)strftime(the_score.day, 9, "%m/%d/%y", localtime(&ct));
-#endif
-
-	/* Save the player name (15 chars) */
-	sprintf(the_score.who, "%-.15s", player_name);
-
-	/* Save the player info XXX XXX XXX */
-	sprintf(the_score.uid, "%7u", player_uid);
-	sprintf(the_score.sex, "%c", (p_ptr->psex ? 'm' : 'f'));
-	sprintf(the_score.p_r, "%2d", p_ptr->prace);
-	sprintf(the_score.p_c, "%2d", p_ptr->pclass);
-
-	/* Save the level and such */
-	sprintf(the_score.cur_lev, "%3d", p_ptr->lev);
-	sprintf(the_score.cur_dun, "%3d", dun_level);
-	sprintf(the_score.max_lev, "%3d", p_ptr->max_plv);
-	sprintf(the_score.max_dun, "%3d", p_ptr->max_dlv);
-
-	/* Save the cause of death (31 chars) */
-	sprintf(the_score.how, "%-.31s", died_from);
-
-
-	/* Lock (for writing) the highscore file, or fail */
-	if (fd_lock(highscore_fd, F_WRLCK)) return (1);
-
-	/* Add a new entry to the score list, see where it went */
-	j = highscore_add(&the_score);
-
-	/* Unlock the highscore file, or fail */
-	if (fd_lock(highscore_fd, F_UNLCK)) return (1);
-
-
 	/* Hack -- Display the top fifteen scores */
-	if (j < 10)
+	if (score_idx < 10)
 	{
-		display_scores_aux(0, 15, j, NULL);
+		display_scores_aux(0, 15, score_idx, NULL);
 	}
 
 	/* Display the scores surrounding the player */
 	else
 	{
-		display_scores_aux(0, 5, j, NULL);
-		display_scores_aux(j - 2, j + 7, j, NULL);
+		display_scores_aux(0, 5, score_idx, NULL);
+		display_scores_aux(score_idx - 2, score_idx + 7, score_idx, NULL);
 	}
 
-
 	/* Success */
-	return (0);
+	return;
 }
 
 
 /*
  * Predict the players location, and display it.
  */
-errr predict_score(void)
+void predict_score(void)
 {
-	int          j;
-
-	high_score   the_score;
+	int j;
 
+	high_score the_score;
 
 	/* No score file */
 	if (highscore_fd < 0)
 	{
-		msg_print("Score file unavailable.");
-		msg_print(NULL);
-		return (0);
+		msgf("Score file unavailable.");
+		message_flush();
+		return;
 	}
 
 
 	/* Save the version */
-	sprintf(the_score.what, "%u.%u.%u",
-	        VERSION_MAJOR, VERSION_MINOR, VERSION_PATCH);
+	strnfmt(the_score.what, 8, "%u.%u.%u",
+			VER_MAJOR, VER_MINOR, VER_PATCH);
 
 	/* Calculate and save the points */
-	sprintf(the_score.pts, "%9lu", (long)total_points());
+	strnfmt(the_score.pts, 10, "%9lu", (unsigned long)total_points());
 
 	/* Save the current gold */
-	sprintf(the_score.gold, "%9lu", (long)p_ptr->au);
+	strnfmt(the_score.gold, 10, "%9lu", (unsigned long)p_ptr->au);
 
 	/* Save the current turn */
-	sprintf(the_score.turns, "%9lu", (long)turn);
+	strnfmt(the_score.turns, 10, "%9lu", (unsigned long)turn);
 
 	/* Hack -- no time needed */
 	strcpy(the_score.day, "TODAY");
 
 	/* Save the player name (15 chars) */
-	sprintf(the_score.who, "%-.15s", player_name);
+	strnfmt(the_score.who, 16, "%-.15s", player_name);
 
 	/* Save the player info XXX XXX XXX */
-	sprintf(the_score.uid, "%7u", player_uid);
-	sprintf(the_score.sex, "%c", (p_ptr->psex ? 'm' : 'f'));
-	sprintf(the_score.p_r, "%2d", p_ptr->prace);
-	sprintf(the_score.p_c, "%2d", p_ptr->pclass);
+	strnfmt(the_score.uid, 8, "%7u", (uint)player_uid);
+	strnfmt(the_score.sex, 2, "%c", (p_ptr->rp.psex ? 'm' : 'f'));
+	strnfmt(the_score.p_r, 3, "%2d", (int)p_ptr->rp.prace);
+	strnfmt(the_score.p_c, 3, "%2d", (int)p_ptr->rp.pclass);
 
 	/* Save the level and such */
-	sprintf(the_score.cur_lev, "%3d", p_ptr->lev);
-	sprintf(the_score.cur_dun, "%3d", dun_level);
-	sprintf(the_score.max_lev, "%3d", p_ptr->max_plv);
-	sprintf(the_score.max_dun, "%3d", p_ptr->max_dlv);
+	strnfmt(the_score.cur_lev, 4, "%3d", p_ptr->lev);
+	strnfmt(the_score.cur_dun, 4, "%3d", p_ptr->depth);
+	strnfmt(the_score.max_lev, 4, "%3d", p_ptr->max_lev);
+	strnfmt(the_score.max_dun, 4, "%3d", p_ptr->max_depth);
 
 	/* Hack -- no cause of death */
 	strcpy(the_score.how, "nobody (yet!)");
@@ -537,61 +657,19 @@ errr predict_score(void)
 		display_scores_aux(0, 5, -1, NULL);
 		display_scores_aux(j - 2, j + 7, j, &the_score);
 	}
-
-
-	/* Success */
-	return (0);
 }
 
 
 
 /*
- * show_highclass - selectively list highscores based on class
- * -KMW-
+ * Selectively list highscores based on class -KMW-
  */
-void show_highclass(int building)
+void show_highclass(void)
 {
-
-	register int i = 0, j, m = 0;
-	int pr, pc, clev/*, al*/;
+	int i = 0, j, m = 0;
+	int pr, pc, clev /*, al */ ;
 	high_score the_score;
-	char buf[1024], out_val[256];
-
-#if 0
-	switch (building)
-	{
-		case 1:
-			prt("               Busts of Greatest Kings", 5, 0);
-			break;
-		case 2:
-			prt("               Plaque - Greatest Arena Champions", 5, 0);
-			break;
-		case 10:
-			prt("               Plaque - Greatest Fighters", 5, 0);
-			break;
-		case 11:
-			prt("               Spires of the Greatest Magic-Users", 5, 0);
-			break;
-		case 12:
-			prt("               Busts of Greatest Priests", 5, 0);
-			break;
-		case 13:
-			prt("               Wall Inscriptions - Greatest Thieves", 5, 0);
-			break;
-		case 14:
-			prt("               Plaque - Greatest Rangers", 5, 0);
-			break;
-		case 15:
-			prt("               Plaque - Greatest Paladins", 5, 0);
-			break;
-		case 16:
-			prt("               Spires of the Greatest Illusionists", 5, 0);
-			break;
-		default:
-			bell();
-			break;
-	}
-#endif
+	char buf[1024];
 
 	/* Build the filename */
 	path_build(buf, 1024, ANGBAND_DIR_APEX, "scores.raw");
@@ -600,8 +678,8 @@ void show_highclass(int building)
 
 	if (highscore_fd < 0)
 	{
-		msg_print("Score file unavailable.");
-		msg_print(NULL);
+		msgf("Score file unavailable.");
+		message_flush();
 		return;
 	}
 
@@ -622,23 +700,21 @@ void show_highclass(int building)
 		pc = atoi(the_score.p_c);
 		clev = atoi(the_score.cur_lev);
 
-		sprintf(out_val, "%3d) %s the %s (Level %2d)",
-		    (m + 1), the_score.who, race_info[pr].title, clev);
-		prt(out_val, (m + 7), 0);
+		prtf(0, m + 7, "%3d) %s the %s (Level %2d)",
+				(m + 1), the_score.who, race_info[pr].title, clev);
 		m++;
 		j++;
 	}
 
-	sprintf(out_val, "You) %s the %s (Level %2d)",
-	    player_name, race_info[p_ptr->prace].title, p_ptr->lev);
-	prt(out_val, (m + 8), 0);
+	prtf(0, m + 8, "You) %s the %s (Level %2d)",
+			player_name, race_info[p_ptr->rp.prace].title, p_ptr->lev);
 
 	(void)fd_close(highscore_fd);
 	highscore_fd = -1;
-	msg_print("Hit any key to continue");
-	msg_print(NULL);
+	msgf("Hit any key to continue");
+	message_flush();
 
-	for (j = 5; j < 18; j++) prt("", j, 0);
+	clear_region(0, 5, 17);
 }
 
 
@@ -648,16 +724,15 @@ void show_highclass(int building)
  */
 void race_score(int race_num)
 {
-	register int i = 0, j, m = 0;
+	int i = 0, j, m = 0;
 	int pr, pc, clev, lastlev;
 	high_score the_score;
-	char buf[1024], out_val[256], tmp_str[80];
+	char buf[1024];
 
 	lastlev = 0;
 
 	/* rr9: TODO - pluralize the race */
-	sprintf(tmp_str,"The Greatest of all the %s", race_info[race_num].title);
-	prt(tmp_str, 5, 15);
+	prtf(15, 5, "The Greatest of all the %s", race_info[race_num].title);
 
 	/* Build the filename */
 	path_build(buf, 1024, ANGBAND_DIR_APEX, "scores.raw");
@@ -666,8 +741,8 @@ void race_score(int race_num)
 
 	if (highscore_fd < 0)
 	{
-		msg_print("Score file unavailable.");
-		msg_print(NULL);
+		msgf("Score file unavailable.");
+		message_flush();
 		return;
 	}
 
@@ -691,10 +766,8 @@ void race_score(int race_num)
 
 		if (pr == race_num)
 		{
-			sprintf(out_val, "%3d) %s the %s (Level %3d)",
-			    (m + 1), the_score.who,
-			race_info[pr].title, clev);
-			prt(out_val, (m + 7), 0);
+			prtf(0, m + 7, "%3d) %s the %s (Level %3d)",
+					(m + 1), the_score.who, race_info[pr].title, clev);
 			m++;
 			lastlev = clev;
 		}
@@ -702,11 +775,10 @@ void race_score(int race_num)
 	}
 
 	/* add player if qualified */
-	if ((p_ptr->prace == race_num) && (p_ptr->lev >= lastlev))
+	if ((p_ptr->rp.prace == race_num) && (p_ptr->lev >= lastlev))
 	{
-		sprintf(out_val, "You) %s the %s (Level %3d)",
-		    player_name, race_info[p_ptr->prace].title, p_ptr->lev);
-		prt(out_val, (m + 8), 0);
+		prtf(0, m + 8, "You) %s the %s (Level %3d)",
+				player_name, race_info[p_ptr->rp.prace].title, p_ptr->lev);
 	}
 
 	(void)fd_close(highscore_fd);
@@ -720,15 +792,14 @@ void race_score(int race_num)
  */
 void race_legends(void)
 {
-	int i, j;
+	int i;
 
 	for (i = 0; i < MAX_RACES; i++)
 	{
 		race_score(i);
-		msg_print("Hit any key to continue");
-		msg_print(NULL);
-		for (j = 5; j < 19; j++)
-			prt("", j, 0);
+		msgf("Hit any key to continue");
+		message_flush();
+        clear_region(0, 5, 18);
 	}
 }
 
@@ -739,16 +810,16 @@ void race_legends(void)
 void kingly(void)
 {
 	/* Hack -- retire in town */
-	dun_level = 0;
+	p_ptr->depth = 0;
 
 	/* Fake death */
-	(void)strcpy(died_from, "Ripe Old Age");
+	(void)strcpy(p_ptr->state.died_from, "Ripe Old Age");
 
 	/* Restore the experience */
 	p_ptr->exp = p_ptr->max_exp;
 
 	/* Restore the level */
-	p_ptr->lev = p_ptr->max_plv;
+	p_ptr->lev = p_ptr->max_lev;
 
 	/* Hack -- Instant Gold */
 	p_ptr->au += 10000000L;
@@ -757,23 +828,24 @@ void kingly(void)
 	Term_clear();
 
 	/* Display a crown */
-	put_str("#", 1, 34);
-	put_str("#####", 2, 32);
-	put_str("#", 3, 34);
-	put_str(",,,  $$$  ,,,", 4, 28);
-	put_str(",,=$   \"$$$$$\"   $=,,", 5, 24);
-	put_str(",$$        $$$        $$,", 6, 22);
-	put_str("*>         <*>         <*", 7, 22);
-	put_str("$$         $$$         $$", 8, 22);
-	put_str("\"$$        $$$        $$\"", 9, 22);
-	put_str("\"$$       $$$       $$\"", 10, 23);
-	put_str("*#########*#########*", 11, 24);
-	put_str("*#########*#########*", 12, 24);
+	put_fstr(22, 1, "             %%%\n"
+					"             %@%\n"
+					"     ___      %      ___\n"
+					"  _=$$###\\   #####	 /###$$=_\n"
+					" $$_______##$$_____$$##_______$$\n"
+					"(**   **   **   **   **   **)\n"
+					"(**   **   **   **   **   **)\n"
+					" TTTTTTTTTTTTTTTTTTTTTTTTTTT\n"
+					" \\. $$$$$$ .\\.  $$$$$$  ./. $$$$$$ ./\n"
+					"  \\  *    |   *   |    *  /\n"
+					"   \\.    .|.     .|.    ./\n"
+					"    ##H##H##H###H##H##H##\n"
+					"    ##H##H##H###H##H##H##\n");
 
 	/* Display a message */
-	put_str("Veni, Vidi, Vici!", 15, 26);
-	put_str("I came, I saw, I conquered!", 16, 21);
-	put_str(format("All Hail the Mighty %s!", sp_ptr->winner), 17, 22);
+	put_fstr(26, 15, "Veni, Vidi, Vici!");
+	put_fstr(21, 16, "I came, I saw, I conquered!");
+	put_fstr(22, 17, "All Hail the Mighty %s!", sp_ptr->winner);
 
 	/* Flush input */
 	flush();
